from typing import Any

from anaplan_sdk import Client
from anaplan_sdk.models.flows import Flow, FlowInput, FlowSummary
from tests.conftest import PyVersionConfig


def test_list_flows(client: Client) -> None:
    flows = client.cw.flows.get_flows()
    assert isinstance(flows, list)
    assert all(isinstance(f, FlowSummary) for f in flows)


def test_list_flows_current_user(client: Client) -> None:
    flows = client.cw.flows.get_flows(current_user_only=True)
    assert isinstance(flows, list)
    assert all(isinstance(f, FlowSummary) for f in flows)


def test_create_flow_pydantic(
    client: Client, flow_pydantic: FlowInput, registry: dict[str, Any]
) -> None:
    flow_id = client.cw.flows.create_flow(flow_pydantic)
    assert flow_id is not None
    registry["flows"].append(flow_id)


def test_create_flow_dict(
    client: Client, flow_dict: dict[str, Any], registry: dict[str, Any]
) -> None:
    flow_id = client.cw.flows.create_flow(flow_dict)
    assert flow_id is not None
    registry["flows"].append(flow_id)


def test_get_flow(client: Client, registry: dict[str, Any]) -> None:
    flow = client.cw.flows.get_flow(registry["flows"][0])
    assert isinstance(flow, Flow)


def test_run_flow(client: Client, config: PyVersionConfig) -> None:
    run_id = client.cw.flows.run_flow(config.test_flow_sync)
    assert run_id is not None


def test_update_flow_pydantic(
    client: Client, flow_pydantic: FlowInput, registry: dict[str, Any]
) -> None:
    client.cw.flows.update_flow(registry["flows"][0], flow_pydantic)


def test_update_flow_dict(
    client: Client, flow_dict: dict[str, Any], registry: dict[str, Any]
) -> None:
    client.cw.flows.update_flow(registry["flows"][1], flow_dict)


def test_delete_flow(client: Client, registry: dict[str, Any]) -> None:
    _ = (client.cw.flows.delete_flow(f) for f in registry["flows"])



import logging
import os
import random
import string
import sys
import time
from dataclasses import dataclass
from random import choices
from typing import Any, Callable

import pytest

import anaplan_sdk.models.cloud_works as cwm
from anaplan_sdk.models.flows import FlowInput, FlowStepInput

logging.basicConfig(level=logging.INFO, format="%(asctime)s [%(levelname)s] %(message)s")
logging.getLogger("httpx").setLevel(logging.ERROR)
logging.getLogger("anaplan_sdk").setLevel(logging.INFO)


@dataclass(slots=True, frozen=True, eq=False)
class PyVersionConfig:
    workspace_id: str
    model_id: str
    alm_model_id_async: str
    alm_src_model_id_async: str
    test_integration_async: str
    test_notification_async: str
    test_flow_async: str
    alm_model_id_sync: str
    alm_src_model_id_sync: str
    test_integration_sync: str
    test_notification_sync: str
    test_flow_sync: str
    connection_id: str = "8b2d5f3a2ff64f13ab52e5b993896386"
    error_run_id: str = "910a68fb814e4225ad683bdafb70ae65"
    scim_user_id: str = "38a0546fd5894c1fac87f8fb71566b3f"


@pytest.fixture(autouse=True)
def random_delay_between_tests():
    yield
    time.sleep(random.uniform(1, 5))  # Avoid hitting rate limit too often


@pytest.fixture(scope="session")
def config() -> PyVersionConfig:
    py_version = f"{sys.version_info.major}.{sys.version_info.minor}"
    workspace_id = "8a81b09d599f3c6e0159f605560c2630"
    if "3.10" in py_version:
        return PyVersionConfig(
            workspace_id=workspace_id,
            model_id="060D1DD6C9D04DD4BF03BA96C1C4B93C",
            alm_model_id_async="174F0F20D5A84FF09C812E95B8E50997",
            alm_src_model_id_async="C852660C6AF547258FB6B97D4942AB27",
            test_integration_async="840ccd8a279a454d99577d9538f24f09",
            test_notification_async="bfe29c0ff7434bde96c94ce1ec1b8e0a",
            test_flow_async="35e19e2f0f594d589f07fd8ba98c30a8",
            alm_model_id_sync="7538F3BE46B94F208C6AF5051919E56E",
            alm_src_model_id_sync="662239743F56420EBECAE7EE0659475C",
            test_integration_sync="44bd0bd4606b4f77b62e70d5ff617f3f",
            test_notification_sync="efc8e3340c054f00bc20dbab1719531f",
            test_flow_sync="8f9a377127844984b12775e9ca072108",
        )
    if "3.11" in py_version:
        return PyVersionConfig(
            workspace_id=workspace_id,
            model_id="D2D29CEE237C422099D83DCC4338CEA6",
            alm_model_id_async="FFBB7D26E61040EBB5CACD80F7F8A01B",
            alm_src_model_id_async="C042E6A3F0334DEC9C826E1E12947789",
            test_integration_async="c0fa795faac047468a59c8dbe3752d75",
            test_notification_async="e2c709c74998460c8688b641cde07cd3",
            test_flow_async="0ca27f18a3f04a1382ecd1745609329b",
            alm_model_id_sync="5C31E832E0B04567B5A8784B25AF3572",
            alm_src_model_id_sync="4FCB1B5224F64339AADC7B322E1663EF",
            test_integration_sync="dfcf1caace4f41748fc589e83c68c65a",
            test_notification_sync="81f0e8c718ed47ada55ce9b88dad548f",
            test_flow_sync="240a3d55754843be824cb4f97e48e061",
        )
    if "3.12" in py_version:
        return PyVersionConfig(
            workspace_id=workspace_id,
            model_id="6EE2A426C16B464193498C1FE28972D1",
            alm_model_id_async="FDB11C592E9445D78337F01BABC89880",
            alm_src_model_id_async="E130519009E54304A54D8D9F1C6D2725",
            test_integration_async="0204ea3261c8431e9e36ff1239c16247",
            test_notification_async="e0f3d33a9c114e3a9c0e0908cffdb5e3",
            test_flow_async="c9fd9841222d43d9886758ba4db4c340",
            alm_model_id_sync="7F51D466D915425CA80BEDF54BF364AE",
            alm_src_model_id_sync="A68B26D67D00443FB8A3428CC83E427D",
            test_integration_sync="18e2c03b0bdf4593bdd964786891ead8",
            test_notification_sync="d90749ebb0e94692b4eaba1963cf0959",
            test_flow_sync="afc6cec5d4b343c6abb4f0b3d41f1f7f",
        )
    if "3.13" in py_version:
        return PyVersionConfig(
            workspace_id=workspace_id,
            model_id="A7A1F7D149054E3080C95C8A085738B5",
            alm_model_id_async="0CBB17195E5D445A9A3692F481D66CBE",
            alm_src_model_id_async="12A4D6B9A816481F83F795F857407049",
            test_integration_async="cf9e1cf27a0f4eddb37a2a4807fd0ffc",
            test_notification_async="e57b5620f006444a9324baaa4bd891ff",
            test_flow_async="c330ca2eda974650bd99fea50b0e3acd",
            alm_model_id_sync="33A804FE8FE34AC291A6E866CA6B2284",
            alm_src_model_id_sync="5327704441464F36B4B05C6BF9DFACD2",
            test_integration_sync="c5bd6fd4b9414ea0959795ccebc8126a",
            test_notification_sync="393d6e91e1874e28baccd7441d28ba36",
            test_flow_sync="61f2a2b05ac64598bfeb6d9eb7eff97a",
        )
    if "3.14" in py_version:
        return PyVersionConfig(
            workspace_id=workspace_id,
            model_id="1D1741C3DD1042A68D8336B328C2324E",
            alm_model_id_async="C48A0D162C964C02964E2EB78190D480",
            alm_src_model_id_async="37DEEDC7D5F2434BBBF836251B90CE23",
            test_integration_async="1eaee08c758e44948805632e766ec0ee",
            test_notification_async="6da7d462645744c39c5b17405daf10c1",
            test_flow_async="e95f6ed1fc2f4c02820edebb919e20b4",
            alm_model_id_sync="06520A761F0B4ECF8AED011A6B7DADCA",
            alm_src_model_id_sync="704A4A635CF1429E91B9C712B848F114",
            test_integration_sync="132704f947044cae8713d9a95da1ba15",
            test_notification_sync="a03de322708b4451b8117544197a8808",
            test_flow_sync="68c07638382244cda4e2d997ddc0b05b",
        )

    logging.fatal(f"Unsupported Python version: {py_version}")
    exit(1)


@pytest.fixture(scope="session")
def model_ids_for_py_version(config: PyVersionConfig) -> tuple[str, str]:
    return config.workspace_id, config.model_id


@pytest.fixture(scope="session")
def list_items_long() -> list[dict[str, Any]]:
    return [{"name": i, "code": i} for i in range(200_000)]  # Force several batches


@pytest.fixture(scope="session")
def list_items_short() -> list[dict[str, Any]]:
    return [{"name": i, "code": i} for i in range(1_000)]  # Single batch


@pytest.fixture
def name() -> str:
    return "Test_" + "".join(choices(string.ascii_uppercase + string.digits, k=12))


@pytest.fixture
def az_blob_connection(name: str) -> cwm.ConnectionInput:
    return cwm.ConnectionInput(
        type="AzureBlob",
        body=cwm.AzureBlobConnectionInput(
            storage_account_name=os.environ["AZ_STORAGE_ACCOUNT"],
            container_name="raw",
            name=name,
            sas_token=os.environ["AZ_STORAGE_SAS_TOKEN"],
        ),
    )


@pytest.fixture
def az_blob_connection_dict(name: str) -> dict[str, Any]:
    return {
        "type": "AzureBlob",
        "body": {
            "storageAccountName": os.environ["AZ_STORAGE_ACCOUNT"],
            "containerName": "raw",
            "name": name,
            "sasToken": os.environ["AZ_STORAGE_SAS_TOKEN"],
        },
    }


@pytest.fixture
def integration_pydantic(name: str, config: PyVersionConfig) -> cwm.IntegrationInput:
    source = cwm.FileSourceInput(
        type="AzureBlob", connection_id=config.connection_id, file="dummy/liquor_sales.csv"
    )
    target = cwm.AnaplanTarget(action_id=112000000064, file_id=113000000055)
    return cwm.IntegrationInput(
        name=name,
        workspace_id=os.environ["ANAPLAN_SDK_TEST_WORKSPACE_ID"],
        model_id=os.environ["ANAPLAN_SDK_TEST_MODEL_ID"],
        jobs=[
            cwm.IntegrationJobInput(type="AzureBlobToAnaplan", sources=[source], targets=[target])
        ],
    )


@pytest.fixture
def multi_step_integration_pydantic(name: str, config: PyVersionConfig) -> cwm.IntegrationInput:
    source = cwm.FileSourceInput(
        type="AzureBlob", connection_id=config.connection_id, file="dummy/liquor_sales.csv"
    )
    target = cwm.AnaplanTarget(action_id=112000000064, file_id=113000000055)
    job = cwm.IntegrationJobInput(type="AzureBlobToAnaplan", sources=[source], targets=[target])
    return cwm.IntegrationInput(
        name=name,
        process_id=118000000012,
        workspace_id=os.environ["ANAPLAN_SDK_TEST_WORKSPACE_ID"],
        model_id=os.environ["ANAPLAN_SDK_TEST_MODEL_ID"],
        jobs=[job, job],
    )


@pytest.fixture
def process_integration_pydantic(name: str) -> cwm.IntegrationProcessInput:
    return cwm.IntegrationProcessInput(
        name=name,
        process_id=118000000012,
        workspace_id=os.environ["ANAPLAN_SDK_TEST_WORKSPACE_ID"],
        model_id=os.environ["ANAPLAN_SDK_TEST_MODEL_ID"],
    )


@pytest.fixture
def integration_dict(name: str, config: PyVersionConfig) -> dict[str, Any]:
    return {
        "name": name,
        "version": "2.0",
        "workspaceId": os.environ["ANAPLAN_SDK_TEST_WORKSPACE_ID"],
        "modelId": os.environ["ANAPLAN_SDK_TEST_MODEL_ID"],
        "nuxVisible": False,
        "jobs": [
            {
                "type": "AzureBlobToAnaplan",
                "sources": [
                    {
                        "connectionId": config.connection_id,
                        "type": "AzureBlob",
                        "file": "dummy/liquor_sales.csv",
                    }
                ],
                "targets": [
                    {"type": "Anaplan", "actionId": "112000000001", "fileId": "112000000001"}
                ],
            }
        ],
    }


@pytest.fixture
def multi_step_integration_dict(name: str, config: PyVersionConfig) -> dict[str, Any]:
    return {
        "name": name,
        "version": "2.0",
        "workspaceId": os.environ["ANAPLAN_SDK_TEST_WORKSPACE_ID"],
        "modelId": os.environ["ANAPLAN_SDK_TEST_MODEL_ID"],
        "processId": "118000000012",
        "nuxVisible": False,
        "jobs": [
            {
                "type": "AzureBlobToAnaplan",
                "sources": [
                    {
                        "connectionId": config.connection_id,
                        "type": "AzureBlob",
                        "file": "dummy/liquor_sales.csv",
                    }
                ],
                "targets": [
                    {"type": "Anaplan", "actionId": "112000000064", "fileId": "113000000055"}
                ],
            },
            {
                "type": "AzureBlobToAnaplan",
                "sources": [
                    {
                        "connectionId": config.connection_id,
                        "type": "AzureBlob",
                        "file": "dummy/liquor_sales.csv",
                    }
                ],
                "targets": [
                    {"type": "Anaplan", "actionId": "112000000064", "fileId": "113000000055"}
                ],
            },
        ],
    }


@pytest.fixture
def process_integration_dict(name: str) -> dict[str, Any]:
    return {
        "name": name,
        "version": "2.0",
        "processId": 118000000012,
        "workspaceId": os.environ["ANAPLAN_SDK_TEST_WORKSPACE_ID"],
        "modelId": os.environ["ANAPLAN_SDK_TEST_MODEL_ID"],
    }


@pytest.fixture
def schedule_pydantic(name: str) -> cwm.ScheduleInput:
    return cwm.ScheduleInput(
        name=name,
        timezone="Europe/Paris",
        start_date="2027-01-01",
        end_date="2029-01-01",
        days_of_week=[0, 2, 5],
        type="daily",
        time="12:00",
    )


@pytest.fixture
def schedule_dict(name: str) -> dict[str, Any]:
    return {
        "name": name,
        "type": "daily",
        "timezone": "Europe/Paris",
        "time": "12:00",
        "daysOfWeek": [0, 2, 5],
        "startDate": "2027-01-01",
        "endDate": "2029-01-01",
    }


@pytest.fixture
def notification_pydantic(name: str) -> cwm.NotificationInput:
    return cwm.NotificationInput(
        integration_ids=[],
        channels=["in_app"],
        notifications=cwm.NotificationConfigInput(
            config=[
                cwm.NotificationItemInput(
                    type="full_failure", users=["8a868cd97f8f98a3017fe45cbdc65e25"]
                )
            ]
        ),
    )


@pytest.fixture
def notification_dict(name: str) -> dict[str, Any]:
    return {
        "integrationIds": [],
        "channels": ["in_app"],
        "notifications": {
            "config": [{"type": "full_failure", "users": ["8a868cd97f8f98a3017fe45cbdc65e25"]}]
        },
    }


@pytest.fixture
def flow_pydantic(name: str) -> FlowInput:
    return FlowInput(
        name=name,
        steps=[
            FlowStepInput(referrer="840ccd8a279a454d99577d9538f24f09"),
            FlowStepInput(
                referrer="c0fa795faac047468a59c8dbe3752d75",
                depends_on=["840ccd8a279a454d99577d9538f24f09"],
            ),
        ],
    )


@pytest.fixture
def flow_dict(name: str) -> dict[str, Any]:
    return {
        "name": name,
        "steps": [
            {"referrer": "840ccd8a279a454d99577d9538f24f09"},
            {
                "referrer": "c0fa795faac047468a59c8dbe3752d75",
                "dependsOn": ["840ccd8a279a454d99577d9538f24f09"],
            },
        ],
    }


@pytest.fixture(scope="session")
def integration_validator() -> Callable[[list[cwm.SingleIntegration]], None]:
    def validate(integrations: list[cwm.SingleIntegration]) -> None:
        assert all(isinstance(i, cwm.SingleIntegration) for i in integrations)
        assert all(
            isinstance(i.jobs, list) and len(i.jobs) > 0 and isinstance(j, cwm.IntegrationJob)
            for i in integrations[1:]
            for j in i.jobs  # pyright: ignore[reportOptionalIterable]
        )
        process, gbq_import, gbq_export, s3_import, s3_export, az_import, az_export = integrations
        assert (
            isinstance(process.process_id, int)
            and (119000000000 > process.process_id >= 118000000000)
            and (process.jobs is None)
        )

        integration_configs = (
            (gbq_import, cwm.TableSource, cwm.AnaplanTarget),
            (gbq_export, cwm.AnaplanSource, cwm.TableTarget),
            (s3_import, cwm.FileSource, cwm.AnaplanTarget),
            (s3_export, cwm.AnaplanSource, cwm.FileTarget),
            (az_import, cwm.FileSource, cwm.AnaplanTarget),
            (az_export, cwm.AnaplanSource, cwm.FileTarget),
        )
        assert all(
            isinstance(integration.jobs[0].sources[0], source_type)  # pyright: ignore
            and isinstance(integration.jobs[0].targets[0], target_type)  # pyright: ignore
            for integration, source_type, target_type in integration_configs
        )

    return validate



from typing import Literal, TypeAlias

from pydantic import Field

from ._base import AnaplanModel


class TaskResultDetail(AnaplanModel):
    local_message_text: str | None = Field(None, description="Error message text.")
    occurrences: int = Field(0, description="The number of occurrences of this error.")
    type: str = Field(description="The type of this error.")
    values: list[str | None] = Field([], description="Further error information if available.")


class TaskResult(AnaplanModel):
    details: list[TaskResultDetail] = Field(
        [], description="The details of this task result if available."
    )
    successful: bool = Field(description="Whether this task completed successfully or not.")
    failure_dump_available: bool = Field(
        description="Whether a failure dump is available for this task."
    )
    nested_results: list["TaskResult"] = Field(
        [], description="The nested results of this task, if available."
    )


class TaskSummary(AnaplanModel):
    id: str = Field(validation_alias="taskId", description="The unique identifier of this task.")
    task_state: Literal["NOT_STARTED", "IN_PROGRESS", "COMPLETE"] = Field(
        description="The state of this task."
    )
    creation_time: int = Field(description="Unix timestamp of when this task was created.")


class Task(TaskSummary):
    task_state: Literal["NOT_STARTED", "IN_PROGRESS"] = Field(description="The state of this task.")  # pyright: ignore[reportIncompatibleVariableOverride]
    progress: float = Field(description="The progress of this task as a float between 0 and 1.")
    current_step: str | None = Field(None, description="The current step of this task.")


class CompletedTask(Task):
    task_state: Literal["COMPLETE"] = Field(description="The state of this task.")  # pyright: ignore[reportIncompatibleVariableOverride]
    result: TaskResult


TaskStatus: TypeAlias = Task | CompletedTask


class _TaskStatusPoll(AnaplanModel):
    task: TaskStatus = Field(discriminator="task_state")


class SyncTaskResult(AnaplanModel):
    source_revision_id: str = Field(description="The ID of the source revision.")
    target_revision_id: str = Field(description="The ID of the target revision.")
    successful: bool = Field(description="Whether the sync task was successful or not.")
    # TODO: Verify if we have results / error details here as well. If so,
    #  add another discriminated union to distinguish between success and failure.


class PendingTask(TaskSummary):
    task_state: Literal["NOT_STARTED", "IN_PROGRESS"] = Field(description="The state of this task.")  # pyright: ignore[reportIncompatibleVariableOverride]
    current_step: str = Field(description="The current step of the sync task.")


class CompletedSyncTask(PendingTask):
    task_state: Literal["COMPLETE"] = Field(description="The state of this task.")  # pyright: ignore[reportIncompatibleVariableOverride]
    result: SyncTaskResult


SyncTask: TypeAlias = PendingTask | CompletedSyncTask


class _SyncTaskStatusPoll(AnaplanModel):
    task: SyncTask = Field(discriminator="task_state")


class ReportTaskResult(SyncTaskResult):
    successful: Literal[True] = Field(description="Whether the sync task was successful or not.")  # pyright: ignore[reportIncompatibleVariableOverride]
    report_file_url: str = Field(
        description="The URL of the report file generated by the sync task."
    )


class ReportTaskError(AnaplanModel):
    title: str = Field(description="The title of the error.")
    message: str = Field(validation_alias="messageText", description="The message of the error.")


class ReportTaskFailureResult(SyncTaskResult):
    successful: Literal[False] = Field(description="Whether the sync task was successful or not.")  # pyright: ignore[reportIncompatibleVariableOverride]
    error: ReportTaskError = Field(description="The error that occurred during the sync task.")


class CompletedReportTask(CompletedSyncTask):
    result: ReportTaskResult | ReportTaskFailureResult = Field(discriminator="successful")  # pyright: ignore[reportIncompatibleVariableOverride]


ReportTask: TypeAlias = PendingTask | CompletedReportTask


class _ReportTaskStatusPoll(AnaplanModel):
    task: ReportTask = Field(discriminator="task_state")



import pytest

from anaplan_sdk import Client, models
from anaplan_sdk.exceptions import (
    AnaplanException,
    InvalidCredentialsException,
    InvalidIdentifierException,
)

test_file = 113000000074
test_action = 118000000028


def test_get_workspace(client: Client) -> None:
    workspace = client.get_workspace()
    assert isinstance(workspace, models.Workspace)
    assert workspace.id == client._workspace_id  # pyright: ignore[reportPrivateUsage]


def test_list_workspaces(client: Client) -> None:
    workspaces, search = client.get_workspaces(), client.get_workspaces("Demo")
    assert isinstance(workspaces, list)
    assert all(isinstance(workspace, models.Workspace) for workspace in workspaces)
    assert all(isinstance(workspace, models.Workspace) for workspace in search)
    assert len(workspaces) > 0
    assert len(search) > 0
    assert len(search) < len(workspaces)


def test_broken_list_files_raises_invalid_identifier_error(broken_client: Client) -> None:
    with pytest.raises(InvalidIdentifierException):
        broken_client.get_files()


def test_unauthenticated_client_raises_exception() -> None:
    with pytest.raises(InvalidCredentialsException):
        _ = Client(user_email="invalid_email", password="pass")


def test_broken_client_alm_raises(broken_client: Client) -> None:
    with pytest.raises(ValueError):
        _ = broken_client.alm


def test_broken_client_transactional_raises(broken_client: Client) -> None:
    with pytest.raises(ValueError):
        _ = broken_client.tr


def test_file_creation_raises_exception(client: Client) -> None:
    with pytest.raises(InvalidIdentifierException):
        client.upload_file(115000000000, b"")


def test_get_model(client: Client) -> None:
    model = client.get_model()
    assert isinstance(model, models.ModelWithTransactionInfo)
    assert model.id == client._model_id  # pyright: ignore[reportPrivateUsage]


def test_list_models(client: Client) -> None:
    _models, current_only, search = (
        client.get_models(),
        client.get_models(True),
        client.get_models(search_pattern="Demo"),
    )
    assert isinstance(_models, list)
    assert all(isinstance(model, models.Model) for model in _models)
    assert all(isinstance(model, models.Model) for model in current_only)
    assert all(isinstance(model, models.Model) for model in search)
    assert len(_models) > len(current_only) > len(search) > 0


def test_list_models_multi_page(client_small_pages: Client) -> None:
    _models = client_small_pages.get_models()
    assert isinstance(_models, list)
    assert len(_models) > 0
    assert all(isinstance(model, models.Model) for model in _models)
    assert len(_models) == len(set(m.id for m in _models))  # Ensure no duplicates when paginating


def test_list_actions(client: Client) -> None:
    actions = client.get_actions()
    assert isinstance(actions, list)
    assert all(isinstance(action, models.Action) for action in actions)
    assert len(actions) > 0


def test_list_files(client: Client) -> None:
    files = client.get_files()
    assert isinstance(files, list)
    assert all(isinstance(file, models.File) for file in files)
    assert len(files) > 0


def test_list_processes(client: Client) -> None:
    processes = client.get_processes()
    assert isinstance(processes, list)
    assert all(isinstance(process, models.Process) for process in processes)
    assert len(processes) > 0


def test_list_imports(client: Client) -> None:
    imports = client.get_imports()
    assert isinstance(imports, list)
    assert all(isinstance(i, models.Import) for i in imports)
    assert len(imports) > 0


def test_list_exports(client: Client) -> None:
    exports = client.get_exports()
    assert isinstance(exports, list)
    assert all(isinstance(e, models.Export) for e in exports)
    assert len(exports) > 0


def test_upload_file_stream(client: Client) -> None:
    client.upload_file_stream(test_file, (str(i) for i in range(10)))
    out = client.get_file(test_file)
    assert out == b"0123456789"


def test_upload_file_async_stream(client: Client) -> None:
    client.upload_file_stream(test_file, (i for i in _async_range(10)))
    out = client.get_file(test_file)
    assert out == b"0123456789"


def test_get_file_stream(client: Client) -> None:
    for chunk in client.get_file_stream(test_file):
        assert isinstance(chunk, bytes)


def test_upload_and_download_file(client: Client) -> None:
    client.upload_file(test_file, "Hi!")
    out = client.get_file(test_file)
    assert out == b"Hi!"


def test_run_process(client: Client) -> None:
    client.run_action(test_action)


def test_list_task_statuses(client: Client) -> None:
    task_statuses = client.get_task_summaries(test_action)
    assert isinstance(task_statuses, list)
    assert all(isinstance(status, models.TaskSummary) for status in task_statuses)
    assert len(task_statuses) > 0


def test_get_task_status(client: Client) -> None:
    task_status = client.get_task_status(test_action, (client.run_action(test_action, False)).id)
    assert isinstance(task_status, models.TaskStatus)


def test_invalid_file_id_raises_exception(client: Client) -> None:
    with pytest.raises(InvalidIdentifierException):
        client.get_file(1)


def test_run_nonexistent_action_raises_exception(client: Client) -> None:
    with pytest.raises(InvalidIdentifierException):
        client.run_action(1)


def test_upload_empty_file(client: Client) -> None:
    with pytest.raises(AnaplanException):
        client.upload_file(test_file, b"")
        # Error occurs here, since the file does not actually exist (only the "address", ID).
        client.get_file(test_file)


def test_with_model(client: Client) -> None:
    other_ws_id = other_model_id = "123"
    other_client = client.with_model(other_model_id, other_ws_id)
    assert isinstance(other_client, Client)
    assert other_client._workspace_id == other_ws_id  # pyright: ignore[reportPrivateUsage]
    assert other_client._model_id == other_model_id  # pyright: ignore[reportPrivateUsage]
    assert other_client.alm._model_id == other_model_id  # pyright: ignore[reportPrivateUsage]
    assert other_client.tr._model_id == other_model_id  # pyright: ignore[reportPrivateUsage]
    assert other_client.alm._model_id == other_model_id  # pyright: ignore[reportPrivateUsage]


def _async_range(count: int):
    for i in range(count):
        yield str(i)



from asyncio import gather
from typing import Any, Callable

import anaplan_sdk.models.cloud_works as cwm
from anaplan_sdk import AsyncClient
from anaplan_sdk.models.cloud_works import Connection, Integration, RunError, RunStatus, RunSummary
from tests.conftest import PyVersionConfig


async def test_list_connections(client: AsyncClient) -> None:
    connections = await client.cw.get_connections()
    assert isinstance(connections, list)
    assert all(isinstance(c, Connection) for c in connections)


async def test_create_connection_pydantic(
    client: AsyncClient, az_blob_connection: cwm.ConnectionInput, registry: dict[str, Any]
) -> None:
    con_id = await client.cw.create_connection(az_blob_connection)
    assert con_id is not None
    registry["connections"].append(con_id)


async def test_create_connection_dict(
    client: AsyncClient, az_blob_connection_dict: dict[str, Any], registry: dict[str, Any]
) -> None:
    con_id = await client.cw.create_connection(az_blob_connection_dict)
    assert con_id is not None
    registry["connections"].append(con_id)


async def test_update_connection_pydantic(
    client: AsyncClient, az_blob_connection: cwm.ConnectionInput, registry: dict[str, Any]
) -> None:
    await client.cw.update_connection(registry["connections"][0], az_blob_connection.body)


async def test_update_connection_dict(
    client: AsyncClient, az_blob_connection_dict: dict[str, Any], registry: dict[str, Any]
) -> None:
    await client.cw.update_connection(registry["connections"][-1], az_blob_connection_dict["body"])


async def test_patch_connection(client: AsyncClient, name: str, registry: dict[str, Any]) -> None:
    await client.cw.patch_connection(registry["connections"][-1], {"name": name})


async def test_get_integration(
    client: AsyncClient,
    test_integration_ids: list[str],
    integration_validator: Callable[[list[cwm.SingleIntegration]], None],
) -> None:
    integrations = await gather(*(client.cw.get_integration(i) for i in test_integration_ids))
    integration_validator(integrations)


async def test_list_integrations(client: AsyncClient) -> None:
    integrations_asc = await client.cw.get_integrations()
    assert isinstance(integrations_asc, list)
    assert all(isinstance(i, Integration) for i in integrations_asc)


async def test_list_integrations_desc(client: AsyncClient) -> None:
    integrations_desc = await client.cw.get_integrations(sort_by="name", descending=True)
    assert isinstance(integrations_desc, list)
    assert all(isinstance(i, Integration) for i in integrations_desc)


async def test_create_integration_pydantic(
    client: AsyncClient, integration_pydantic: cwm.IntegrationInput, registry: dict[str, Any]
) -> None:
    integration_id = await client.cw.create_integration(integration_pydantic)
    assert integration_id is not None
    registry["integrations"].append(integration_id)


async def test_create_multi_step_integration_pydantic(
    client: AsyncClient,
    multi_step_integration_pydantic: cwm.IntegrationInput,
    registry: dict[str, Any],
) -> None:
    integration_id = await client.cw.create_integration(multi_step_integration_pydantic)
    assert integration_id is not None
    registry["integrations"].append(integration_id)


async def test_create_process_integration_pydantic(
    client: AsyncClient,
    process_integration_pydantic: cwm.IntegrationProcessInput,
    registry: dict[str, Any],
) -> None:
    integration_id = await client.cw.create_integration(process_integration_pydantic)
    assert integration_id is not None
    registry["integrations"].append(integration_id)


async def test_update_integration_pydantic(
    client: AsyncClient, integration_pydantic: cwm.IntegrationInput, registry: dict[str, Any]
) -> None:
    await client.cw.update_integration(registry["integrations"][0], integration_pydantic)


async def test_update_multi_step_integration_pydantic(
    client: AsyncClient,
    registry: dict[str, Any],
    multi_step_integration_pydantic: cwm.IntegrationInput,
) -> None:
    await client.cw.update_integration(registry["integrations"][1], multi_step_integration_pydantic)


async def test_update_process_integration_pydantic(
    client: AsyncClient,
    registry: dict[str, Any],
    process_integration_pydantic: cwm.IntegrationProcessInput,
) -> None:
    await client.cw.update_integration(registry["integrations"][-1], process_integration_pydantic)


async def test_create_integration_dict(
    client: AsyncClient, integration_dict: dict[str, Any], registry: dict[str, Any]
) -> None:
    integration_id = await client.cw.create_integration(integration_dict)
    assert integration_id is not None
    registry["integrations"].append(integration_id)


async def test_create_multi_step_integration_dicts(
    client: AsyncClient, multi_step_integration_dict: dict[str, Any], registry: dict[str, Any]
) -> None:
    integration_id = await client.cw.create_integration(multi_step_integration_dict)
    assert integration_id is not None
    registry["integrations"].append(integration_id)


async def test_create_process_integration_dicts(
    client: AsyncClient, multi_step_integration_dict: dict[str, Any], registry: dict[str, Any]
) -> None:
    integration_id = await client.cw.create_integration(multi_step_integration_dict)
    assert integration_id is not None
    registry["integrations"].append(integration_id)


async def test_update_integration_dict(
    client: AsyncClient, integration_dict: dict[str, Any], registry: dict[str, Any]
) -> None:
    await client.cw.update_integration(registry["integrations"][-3], integration_dict)


async def test_update_multi_step_integration_dict(
    client: AsyncClient, registry: dict[str, Any], multi_step_integration_dict: dict[str, Any]
) -> None:
    await client.cw.update_integration(registry["integrations"][-2], multi_step_integration_dict)


async def test_update_process_integration_dict(
    client: AsyncClient,
    registry: dict[str, Any],
    process_integration_pydantic: cwm.IntegrationProcessInput,
) -> None:
    await client.cw.update_integration(registry["integrations"][-1], process_integration_pydantic)


async def test_run_integration(
    client: AsyncClient, config: PyVersionConfig, registry: dict[str, Any]
) -> None:
    run_id = await client.cw.run_integration(config.test_integration_async)
    assert run_id is not None
    registry["run_id"] = run_id


async def test_get_run_history(client: AsyncClient, registry: dict[str, Any]) -> None:
    history = await client.cw.get_run_history(registry["integrations"][-1])
    assert isinstance(history, list)
    assert all((isinstance(i, RunSummary) for i in history))


async def test_get_run_status(client: AsyncClient, registry: dict[str, Any]) -> None:
    status = await client.cw.get_run_status(registry["run_id"])
    assert isinstance(status, RunStatus)


async def test_create_schedule_pydantic(
    client: AsyncClient, registry: dict[str, Any], schedule_pydantic: cwm.ScheduleInput
) -> None:
    await client.cw.create_schedule(registry["integrations"][0], schedule_pydantic)


async def test_update_schedule_pydantic(
    client: AsyncClient, registry: dict[str, Any], schedule_pydantic: cwm.ScheduleInput
) -> None:
    await client.cw.update_schedule(registry["integrations"][0], schedule_pydantic)


async def test_set_schedule_enabled(client: AsyncClient, registry: dict[str, Any]) -> None:
    await client.cw.set_schedule_status(registry["integrations"][0], "enabled")


async def test_set_schedule_disabled(client: AsyncClient, registry: dict[str, Any]) -> None:
    await client.cw.set_schedule_status(registry["integrations"][0], "disabled")


async def test_delete_schedule(client: AsyncClient, registry: dict[str, Any]) -> None:
    await client.cw.delete_schedule(registry["integrations"][0])


async def test_create_schedule_dict(
    client: AsyncClient, registry: dict[str, Any], schedule_dict: dict[str, Any]
) -> None:
    await client.cw.create_schedule(registry["integrations"][0], schedule_dict)


async def test_update_schedule_dict(
    client: AsyncClient, registry: dict[str, Any], schedule_dict: dict[str, Any]
) -> None:
    await client.cw.update_schedule(registry["integrations"][0], schedule_dict)


async def test_update_notification_dict(
    client: AsyncClient, config: PyVersionConfig, notification_dict: dict[str, Any]
) -> None:
    notification_dict["integrationIds"] = [config.test_integration_async]
    await client.cw.update_notification_config(config.test_notification_async, notification_dict)


async def test_delete_notification(client: AsyncClient, registry: dict[str, Any]) -> None:
    await client.cw.delete_notification_config(integration_id=registry["integrations"][0])


async def test_create_notification_dict(
    client: AsyncClient, notification_dict: dict[str, Any], registry: dict[str, Any]
) -> None:
    notification_dict["integrationIds"] = [registry["integrations"][0]]
    await client.cw.create_notification_config(notification_dict)


async def test_get_notification(client: AsyncClient, config: PyVersionConfig) -> None:
    notification = await client.cw.get_notification_config(config.test_notification_sync)
    assert isinstance(notification, cwm.NotificationConfig)


async def test_get_notification_by_integration(
    client: AsyncClient, registry: dict[str, Any]
) -> None:
    notification = await client.cw.get_notification_config(
        integration_id=registry["integrations"][0]
    )
    assert isinstance(notification, cwm.NotificationConfig)


async def test_get_run_error(client: AsyncClient, config: PyVersionConfig) -> None:
    run_error = await client.cw.get_run_error(config.error_run_id)
    assert isinstance(run_error, RunError)


async def test_delete_integration(client: AsyncClient, registry: dict[str, Any]) -> None:
    await gather(*(client.cw.delete_integration(i) for i in registry["integrations"]))


async def test_delete_connection(client: AsyncClient, registry: dict[str, Any]) -> None:
    await gather(*(client.cw.delete_connection(c) for c in registry["connections"]))



from asyncio import gather

from anaplan_sdk import AsyncClient
from anaplan_sdk.models import User


async def test_list_users(client: AsyncClient) -> None:
    users, search = await gather(client.audit.get_users(), client.audit.get_users("vinzenz"))
    assert isinstance(users, list)
    assert all(isinstance(user, User) for user in users)
    assert all(isinstance(user, User) for user in search)
    assert len(users) > 0
    assert len(search) > 0
    assert len(search) < len(users)


async def test_get_user(client: AsyncClient) -> None:
    user = await client.audit.get_user()
    assert isinstance(user, User)


async def test_events(client: AsyncClient) -> None:
    events = await client.audit.get_events(1)
    assert isinstance(events, list)



import os

import pytest

from anaplan_sdk import Client
from tests.conftest import PyVersionConfig


@pytest.fixture(scope="session")
def client(config: PyVersionConfig) -> Client:
    return Client(
        workspace_id=config.workspace_id,
        model_id=config.model_id,
        certificate=os.environ["ANAPLAN_SDK_TEST_CERT"],
        private_key=os.environ["ANAPLAN_SDK_TEST_PK"],
        retry_count=4,
        backoff=5,
        timeout=120,
    )


@pytest.fixture(scope="session")
def client_small_pages(config: PyVersionConfig) -> Client:
    return Client(
        workspace_id=config.workspace_id,
        model_id=config.model_id,
        certificate=os.environ["ANAPLAN_SDK_TEST_CERT"],
        private_key=os.environ["ANAPLAN_SDK_TEST_PK"],
        page_size=100,
        retry_count=3,
        backoff=5,
        timeout=120,
    )


@pytest.fixture(scope="session")
def alm_client(client: Client, config: PyVersionConfig) -> Client:
    return client.with_model(config.alm_model_id_sync)


@pytest.fixture(scope="session")
def alm_src_client(client: Client, config: PyVersionConfig) -> Client:
    return client.with_model(config.alm_src_model_id_sync)


@pytest.fixture(scope="session")
def test_integration_ids(config: PyVersionConfig) -> list[str]:
    return [
        config.test_integration_sync,
        "040a5b1572d74e14adbbc8bada248f41",
        "7839b6bd6c0649acad13cf9d38a374f6",
        "fc24f171c6ba49b49b19b4502f98e62d",
        "a1b5719444444e678eb3b6bbcc137e1a",
        "d1108b906b9c4447b5a62af8f2938a5d",
        "7fede57c307b46d394b892456bc2083e",
    ]


@pytest.fixture(scope="session")
def broken_client() -> Client:
    return Client(
        workspace_id="",
        model_id="",
        certificate=os.environ["ANAPLAN_SDK_TEST_CERT"],
        private_key=os.environ["ANAPLAN_SDK_TEST_PK"],
        retry_count=1,
    )


@pytest.fixture(scope="session")
def registry() -> dict[str, list[dict[str, str]] | None]:
    return {"connections": [], "integrations": [], "flows": [], "run_id": None}
